/*

"Show Keys"
version 1.0
====================================================================================================

This plugin shows which of keys (forward, moveleft, moveright, back, jump, duck)
are in use every frame and prints it on hud.
It also shows horizontal speed of player.
   
"Show Keys" plugin is useful for making kreedz demos/guides,
or analising your own jumps.
   
I recommened you to use "Show Keys" only on lan servers with low ping.
   
   
====================================================================================================
Server cvars:
====================================================================================================
   
keys_channel 4
- This will sets which hud channel to use.
Possible channels are 1, 2, 3 or 4.
Change this value, if you do not see some things on hud,
which should be shown by other plugins.


====================================================================================================
Client commands:
====================================================================================================
 
say /klawisz
- This will turn on/off all what this plugin does.

pokaz_klawisz <on|off>
- The same.


pozycja_klawiszy <Float: x> <Float: y>
-This will set the position of shown hud text.
       
klawisz_kolor <red> <green> <blue>
- This will set color of hud text
       
klawisz_szybkosc <on|off>
- This will turn on/off horizontal speed meter.


====================================================================================================
Client commands for esier demos naming:
====================================================================================================

* nagraj_plik <filename>

Those things will be changed in <filename>:

%map	-	The map you are in
%nick	-	Your nick

%year	-	Current year (server time)
%month
%day
%hour
%minute
%second

%num	-	After use in `nagraj_plik <filename>`, it will be 1.
		After 1st use of command `nagraj_nowy` it will become 2
		After 2nd use of command `nagraj_nowy` it will become 2
		And so on.

* nagraj_znow

	- The name of demo wont be changed. It will start a new record to current file.

* nagraj_nowy

	- The name of demo will be reformated. 
	%second, %minute, %hour and %num may change.

Example:

	nagraj_plik %map-%nick-%num
		- Starts recording demo to "de_dust-MyNick-1"
	
	nagraj_znow
		- Starts recording to the same file "de_dust-MyNick-1"
		(overwrites the old one)
	
	nagraj_nowy
		- Starts recording demo to new file "de_dust-MyNick-2"
		

%num will be set to 1 every time when you run `nagraj_plik <filename>`
And it will be encreased by 1 every time of running `nagraj_nowy`


====================================================================================================
Required modules:
====================================================================================================
   
fakemeta


====================================================================================================
*/


#include <amxmodx>
#include <fakemeta>

#define PLUGIN "Show Keys"
#define VERSION "1.0"
#define AUTHOR "coderiz"


// cvar pointers
new cvar_keys_channel


// server settings
new g_srv_channel

// player settings
new bool:g_plr_showkeys[32]

new Float:g_plr_hudpos[32][2]
new g_plr_hudcolor[32][3]
new bool:g_plr_showspeed[32]

new g_plr_recfile[32][64]
new g_plr_recfile_pre[32][64]
new g_plr_recnum[32]


// Main stuff
// -------------------------------------------------------------------------------------------------

public plugin_init() {
	register_plugin(PLUGIN, VERSION, AUTHOR)
	
	
	
	cvar_keys_channel = register_cvar ( "keys_channel", "4", FCVAR_SERVER)
	
	g_srv_channel = get_pcvar_num ( cvar_keys_channel )
	if(g_srv_channel < 1 || g_srv_channel > 4)
	{
		g_srv_channel = 4
	}
	
	register_clcmd("say /klawisz","cl_ShowKeys")
	register_clcmd("pokaz_klawisz","cl_ShowKeys")
	register_clcmd("pozycja_klawiszy","cl_opt_HudPos")
	register_clcmd("klawisz_kolor","cl_opt_HudColor")
	register_clcmd("klawisz_szybkosc","cl_opt_ShowSpeed")

	register_clcmd("nagraj_plik","cl_record")
	register_clcmd("nagraj_znow","cl_rerecord")
	register_clcmd("nagraj_nowy","cl_newrecord")
	
	
	register_forward(FM_PlayerPreThink, "fwd_PlayerPreThink")
	register_forward(FM_ClientPutInServer, "fwd_ClientPutInServer")
	
	// Add your code here...
}


// Forwards
// -------------------------------------------------------------------------------------------------

public fwd_ClientPutInServer(id)
{
	g_plr_showkeys[id-1] = false
	
	g_plr_hudpos[id-1][0] = 0.8
	g_plr_hudpos[id-1][1] = 0.01
	
	g_plr_hudcolor[id-1][0] = 150
	g_plr_hudcolor[id-1][1] = 0
	g_plr_hudcolor[id-1][2] = 0
	
	g_plr_showspeed[id-1] = true
	
	g_plr_recfile[id-1] = ""
	g_plr_recfile_pre[id-1] = ""
	g_plr_recnum[id-1] = 1
}

public fwd_PlayerPreThink(id)
{
	if(is_user_connected(id) && is_user_alive(id) && g_plr_showkeys[id-1])
	{
		new buttons = pev(id, pev_button)
		new kw[6], ka[6], ks[6], kd[6], kjump[10], kduck[10]
		
		
		if(buttons & IN_FORWARD)
		{
			kw = "[w]"
		}
		else
		{
			kw = "[-]"
		}
		
		if(buttons & IN_MOVELEFT)
		{
			ka = "[a]"
		}
		else
		{
			ka = "[-]"
		}
		
		if(buttons & IN_MOVERIGHT)
		{
			kd = "[d]"
		}
		else
		{
			kd = "[-]"
		}
		
		if(buttons & IN_BACK)
		{
			ks = "[s]"
		}
		else
		{
			ks = "[-]"
		}
		
		if(buttons & IN_JUMP)
		{
			kjump = "[Jump]"
		}
		else
		{
			kjump = "[----]"
		}
		
		if(buttons & IN_DUCK)
		{
			kduck = "[Duck]"
		}
		else
		{
			kduck = "[----]"
		}
		

		
		set_hudmessage(g_plr_hudcolor[id-1][0], g_plr_hudcolor[id-1][1], g_plr_hudcolor[id-1][2],
		g_plr_hudpos[id-1][0], g_plr_hudpos[id-1][1], 0, 2.5, 0.0, 0.0, 0.1,
		g_srv_channel)
		
		if(g_plr_showspeed[id-1])
		{
			new Float:speed[3]
			new speedstr[6]

			pev(id, pev_velocity, speed)
		
			float_to_str ( 
			floatsqroot(floatadd(floatpower(speed[0], 2.0), floatpower(speed[1], 2.0)))
			, speedstr, 5)

			show_hudmessage(id, "   %s ^n%s %s %s ^n%s  %s^n^nSpeed: %s", 
			kw, ka, ks, kd, kduck, kjump,
			speedstr)
		}
		else
		{
			show_hudmessage(id, "   %s ^n%s %s %s ^n%s  %s", 
			kw, ka, ks, kd, kduck, kjump
			)			
		}
	}
	
	return 1
}


// Client functions
// -------------------------------------------------------------------------------------------------

public cl_ShowKeys (id)
{

	new arg[10]
	read_argv(0, arg, 9)
	
	if(equal(arg, "show_keys"))
	{
		read_argv(1, arg, 9)
		
		if(equal(arg, "on") || equal(arg, "1"))
		{
			g_plr_showkeys[id-1] = true
		} else if(equal(arg, "off") || equal(arg, "0"))
		{
			g_plr_showkeys[id-1] = false
		} else {
			client_print(id, print_console, "Uzyj: pokaz_klawisz <on|off>")
		}
		
		return 1
	}
	
	
	if(!is_user_connected(id))
	{
		g_plr_showkeys[id-1] = false
		return 1
	}
	
	if(g_plr_showkeys[id-1])
	{
		g_plr_showkeys[id-1] = false
		return 1
	}
	else
	{
		g_plr_showkeys[id-1] = true
		return 1	
	}
	
	return 1
}

public cl_opt_HudPos(id)
{
	if(read_argc() != 3)
	{
		client_print(id, print_console, "Uzyj: pozycja_klawiszy <Pos x> <pos y>")
		client_print(id, print_console, "   Domyslna wartosc:")
		client_print(id, print_console, "   <Pos x> = 0.8")
		client_print(id, print_console, "   <Pos y> = 0.01")
		
		return 1
	}
	
	new Float:xf, Float:yf
	new xstr[10], ystr[10]

	read_argv(1, xstr, 9)
	read_argv(2, ystr, 9)
	
	xf = str_to_float(xstr)
	yf = str_to_float(ystr)
	
	if( ( (xf < 1.0 && xf > 0.0) || xf == -1.0) && ( (yf < 1.0 && yf > 0.0) || yf == -1.0) )
	{
		g_plr_hudpos[id-1][0] = xf
		g_plr_hudpos[id-1][1] = yf	
		return 1
	}
	else
	{
		client_print(id, print_console, "<Pos x> i <Pos y> musisz uruchomic punkt wartosci")
		client_print(id, print_console, " pomiedzy 0.0 a 1.0 i rowny -1.0 ktory znajduje sie na srodku.")	
		return 1
	}
	
	return 1
}

public cl_opt_HudColor(id)
{
	if(read_argc() != 4)
	{
		client_print(id, print_console, "Uzyj: klawisz_kolor <red> <green> <blue>")
		client_print(id, print_console, "   Domysla wartosc:")
		client_print(id, print_console, "   <red> = 150")
		client_print(id, print_console, "   <green> = 0")	
		client_print(id, print_console, "   <blue> = 0")
		
		return 1
	}
	
	new r, g, b
	new rstr[4], gstr[4], bstr[4]

	read_argv(1, rstr, 3)
	read_argv(2, gstr, 3)
	read_argv(3, bstr, 3)
	
	r = str_to_num(rstr)
	g = str_to_num(gstr)
	b = str_to_num(bstr)
	

	
	if(r<=255 && r>=0 && g<=255 && g>=0 && b<=255 && b>=0)
	{
		g_plr_hudcolor[id-1][0] = r
		g_plr_hudcolor[id-1][1] = g
		g_plr_hudcolor[id-1][2] = b
	}
	else
	{
		client_print(id, print_console, "<red>, <green> i <blue> must be integers from interval [0;255]")
	}
	
	return 1
}

public cl_opt_ShowSpeed(id)
{
	if(read_argc() != 2)
	{
		client_print(id, print_console, "Uzyj: klawisz_szybkosc <on/off>")
		return 1
	}
	
	new arg[4]

	read_argv(1, arg, 3)
	
	if(equal(arg, "on") || equal(arg, "1"))
	{
		g_plr_showspeed[id-1] = true
	}
	else if(equal(arg, "off") || equal(arg, "0"))
	{
		g_plr_showspeed[id-1] = false
	} else
	{
		client_print(id, print_console, "Uzyj: klawisz_szybkosc <on/off>")
	}
	
	return 1
}

public cl_record(id)
{
	if(read_argc() != 2)
	{
		client_print(id, print_console, "Uzyj: nagraj_plik <filename>")
		return 1
	}
	
	read_argv(1, g_plr_recfile[id-1], 63)
	
	g_plr_recfile_pre[id-1] = g_plr_recfile[id-1]
	
	
	g_plr_recnum[id-1] = 1

	format_filename(g_plr_recfile[id-1], 63, id)
	
	client_cmd(id, "stop")
	client_cmd(id, "record %s", g_plr_recfile[id-1])
	
	return 1
}

public cl_rerecord(id)
{
	if(strlen(g_plr_recfile[id-1]) < 1)
	{
		client_print(id, print_console, "Niema takiego pliku. Uzyj: nagraj_plik <filename> pierwszy.")
	}
	else
	{
		client_cmd(id, "stop")
		client_cmd(id, "record %s", g_plr_recfile[id-1])
	}
	
	return 1
		
}

public cl_newrecord(id)
{
	if(strlen(g_plr_recfile_pre[id-1]) < 1)
	{
		client_print(id, print_console, "Niema takiego pliku. Uzyj: nagraj_plik <filename> pierwszy.")
	}
	else
	{
		g_plr_recfile[id-1] = g_plr_recfile_pre[id-1]
		
		format_filename(g_plr_recfile[id-1], 63, id)
		
		client_cmd(id, "stop")
		client_cmd(id, "record %s", g_plr_recfile[id-1])
	}
	
	return 1
}

//
// -------------------------------------------------------------------------------------------------

public format_filename(filename[], len, id)
{
	new mapname[32], plrname[32]

	get_mapname (mapname, 31)
	get_user_name(id, plrname, 31)


	new t = get_systime()
	new year[5], month[3], day[3], hour[3], minute[3], second[3]
	
	format_time(year, 4, "%Y", t)
	format_time(month, 2, "%m", t)
	format_time(day, 2, "%d", t)
	format_time(hour, 2, "%H", t)
	format_time(minute, 2, "%M", t)
	format_time(second, 2, "%S", t)

	
	
	replace_all(filename, 63, "%map", mapname)
	replace_all(filename, 63, "%nick", plrname)

	replace_all(filename, 63, "%year", year)
	replace_all(filename, 63, "%month", month)
	replace_all(filename, 63, "%day", day)
	replace_all(filename, 63, "%hour", hour)
	replace_all(filename, 63, "%minute", minute)
	replace_all(filename, 63, "%second", second)
	
	if(strfind(filename, "%num"))
	{
		new numstr[5]
		num_to_str(g_plr_recnum[id-1]++, numstr, 4)
		replace_all(filename, 63, "%num", numstr)
	}
}
